"""Constants used by yggdrasil."""


LANG2EXT = {'python': '.py',
            'matlab': '.m',
            'r': '.R',
            'c': '.c',
            'cpp': '.cpp',
            'c++': '.cpp',
            'fortran': '.f90',
            'yaml': '.yml'}
EXT2LANG = {v: k for k, v in LANG2EXT.items()}
